<?php

class Mlumen_Api_Blog {
	/**
	 * @var string
	 */
	private $api_url;

	public $data = array(
		'active' => null,
		'token'  => null,
		'email'  => null
	);

	/**
	 * @var string
	 */
	public $error_message;

	/**
	 * Set API url
	 */
	public function __construct( string $api_url ) {
		$this->api_url = $api_url;
	}

	/**
	 * Generate guid
	 *
	 * @return string
	 */
	public function generate_guid() {
		return sprintf( '%04X%04X-%04X-%04X-%04X-%04X%04X%04X',
			mt_rand( 0, 65535 ),
			mt_rand( 0, 65535 ),
			mt_rand( 0, 65535 ),
			mt_rand( 16384, 20479 ),
			mt_rand( 32768, 49151 ),
			mt_rand( 0, 65535 ),
			mt_rand( 0, 65535 ),
			mt_rand( 0, 65535 )
		);
	}

	/**
	 * Add blog
	 *
	 * @return boolean
	 */
	public function add_blog( string $email ) {
		$verification_file = '/' . strtolower( $this->generate_guid() ) . '.mlumen';
		$verification_file_name = WP_CONTENT_DIR . $verification_file;
		file_put_contents( $verification_file_name, get_site_url() );

		if ( ! file_exists( $verification_file_name ) ) {
			$this->error_message = __( 'Błąd zapisu pliku weryfikacyjnego', 'mlumen' );

			return false;
		}

		$verification_file_path = str_replace( get_site_url(), '', content_url( $verification_file ) );
		$parameters = array(
			'acceptedRodoPrivacyPolicy'  => true,
			'acceptedTermsAndConditions' => true,
			'contactMail'                => $email,
			'url'                        => get_site_url(),
			'verificationFileName'       => $verification_file_path
		);

		$response = wp_remote_post( $this->api_url . '/blogs', array(
			'headers'     => array( 'Content-Type' => 'application/json' ),
			'body'        => json_encode( $parameters ),
			'method'      => 'POST',
			'data_format' => 'body',
		) );

		// verification file
		unlink( $verification_file_name );

		if ( is_wp_error( $response ) ) {
			$this->error_message = __( 'Błąd połaczenia', 'mlumen' ) . ' ' . $response->get_error_message();

			return false;
		}

		$body = json_decode( $response['body'] );
		$http_code = wp_remote_retrieve_response_code( $response );
		if ( $http_code !== 201 ) {
			return $this->display_error_message( $response );
		}

		if ( ! isset( $body->token ) ) {
			$this->error_message = __( 'Błąd aktywacji', 'mlumen' );

			return false;
		}

		$this->data = array(
			'active' => true,
			'token'  => $body->token,
			'email'  => $email
		);

		return true;
	}

	/**
	 * Change e-mail
	 *
	 * @return boolean
	 */
	public function change_email( string $email, string $token ) {
		$parameters = array(
			'email' => $email,
		);
		$response = wp_remote_post( $this->api_url . '/blogs/' . $token, array(
			'headers'     => array( 'Content-Type' => 'application/json' ),
			'body'        => json_encode( $parameters ),
			'method'      => 'PATCH',
			'data_format' => 'body',
		) );

		if ( is_wp_error( $response ) ) {
			$this->error_message = __( 'Błąd połaczenia', 'mlumen' ) . ' ' . $response->get_error_message();

			return false;
		}

		$http_code = wp_remote_retrieve_response_code( $response );
		if ( $http_code !== 200 ) {
			return $this->display_error_message( $response );
		}

		$this->data['email'] = $email;

		return true;
	}


	/**
	 * Check connection
	 *
	 * @return boolean
	 */
	public function check_connection( string $url ) {
		$parameters = array(
			'url' => $url,
		);
		$response   = wp_remote_post( $this->api_url . '/blogs/test', array(
			'headers'     => array( 'Content-Type' => 'application/json' ),
			'body'        => json_encode( $parameters ),
			'method'      => 'POST',
			'data_format' => 'body',
		) );

		if ( is_wp_error( $response ) ) {
			$this->error_message = __( 'Błąd połaczenia', 'mlumen' ) . ' ' . $response->get_error_message();

			return false;
		}

		$http_code = wp_remote_retrieve_response_code( $response );
		if ( $http_code !== 200 ) {
			return $this->display_error_message( $response );
		}

		return true;
	}


	/**
	 * Delete blog
	 *
	 * @return boolean
	 */
	public function delete_blog( string $token ) {
		$response = wp_remote_request( $this->api_url . '/blogs/' . $token, array(
			'headers' => array( 'Content-Type' => 'application/json' ),
			'method'  => 'DELETE',
		) );

		if ( is_wp_error( $response ) ) {
			$this->error_message = __( 'Błąd połaczenia', 'mlumen' ) . ' ' . $response->get_error_message();

			return false;
		}

		$http_code = wp_remote_retrieve_response_code( $response );
		if ( ! in_array( $http_code, array( 200, 404 ) ) ) {
			return $this->display_error_message( $response );
		}

		return true;
	}

	/**
	 * Display error
	 *
	 * @return boolean
	 */
	private function display_error_message( array $response ) {
		$body = json_decode( $response['body'] );
		$http_code = wp_remote_retrieve_response_code( $response );
		$http_message = wp_remote_retrieve_response_message( $response );

		if ( ! empty( $body->message ) ) {
			$this->error_message = $body->message . ' ( http code ' . $http_code . ' )';
		} elseif ( ! empty( $http_message ) ) {
			$this->error_message = $http_message . ' ( http code ' . $http_code . ' )';
		} else {
			$this->error_message = __( 'Błąd połaczenia', 'mlumen' ) . ' ( http code ' . $http_code . ' )';
		}

		return false;
	}


}